/*
** Jo Sega Saturn Engine
** Copyright (c) 2012-2016, Johannes Fetz (johannesfetz@gmail.com)
** All rights reserved.
**
** Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are met:
**     * Redistributions of source code must retain the above copyright
**       notice, this list of conditions and the following disclaimer.
**     * Redistributions in binary form must reproduce the above copyright
**       notice, this list of conditions and the following disclaimer in the
**       documentation and/or other materials provided with the distribution.
**     * Neither the name of the Johannes Fetz nor the
**       names of its contributors may be used to endorse or promote products
**       derived from this software without specific prior written permission.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
** ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
** DISCLAIMED. IN NO EVENT SHALL Johannes Fetz BE LIABLE FOR ANY
** DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
** (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
** LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
** ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

/****************************
Mini-FPS style collision detection demo by XL2
This is by no-means complete nor bug-free.
This is just a quick demo to show how the collision tool can be used.
The collision tool is quite buggy, and this demo was done QUICKLY, with very little testing,
so there are bugs for sure.
The coordinate system is the opposite of Sonic Z-Treme (since now we are the camera, vs Sonic moving independant of the camera), so
I couldn't fully adapt what worked in Sonic Z-Treme.
The collision tool isn't very good with slopes as it sometimes merges them with straight walls/floor, I will have to improve it.
The FPS movement isn't perfect, there is a problem with Sin/Cos where you
go faster one direction. Cause unknown, could be weird overflow bug.

TODO : Correct bugs
TODO (maybe) : Add damage, firing and ennemies (probably no AI, we'll see. Sprites or 3D models, I'm not sure yet as I like the idea of having 30 ennemies on screen, which is impossible with 3D. Maybe a mix ->At distance, sprite, up close, low-poly 3D...)
TODO : Add analog controls
TODO : Add mipmapping and display of low-poly meshes at a distance (we'll see if it works well enough)
TODO (MAYBE) : Add split-screen
WISHFUL (NOT LIKELY) TODO (but certainly not by me since I know nothing about network protocols and all) : Add Netlink multiplayer
(could even make it go straight on the Internet since by using DreamPi you have a stable speed that you didn't have back in 1997)

*****************************/
/*#define LWRAM_LOW (0x00200000)
#define LWRAM_HIGH (0x002FFFFF)*/

#include <jo/jo.h>
#include "Player.h"

#include "Display.h"
#include "Control.h"
#include "Collision.h"




void _reset(_player *currentPlayer)
{
	currentPlayer->XSP = 0;
	currentPlayer->YSP = 0;
	currentPlayer->ZSP = 0;
	currentPlayer->angle_increment = 0.0;
    currentPlayer->x = 0;
    currentPlayer->y = -150;
    currentPlayer->z = 0;
    currentPlayer->ry = 180;
    currentPlayer->rx = 0;
    currentPlayer->rz = 0;
    currentPlayer->vertical_angle = 0;
    currentPlayer->is_in_air = false;
    currentPlayer->movement_speed = 0;
    currentPlayer->lateral_speed = 0;
    currentPlayer->crouchH = 0;
    currentPlayer->is_crouching=false;

    int i = 2;
    currentPlayer->owned_weapons[0] = true; currentPlayer->owned_weapons[1] = true;
    currentPlayer->weapon=1; currentPlayer->ammo[1]= 30; currentPlayer->ammo[0]=1;
    for (i = 2; i<9; i++)
    {
        currentPlayer->ammo[i] = 0;
        currentPlayer->owned_weapons[i] = false;
    }
    currentPlayer->is_firing=0;

    jo_audio_play_cd_track(2, 2, 1);
}

void update_positions(_player *currentPlayer)
{
        currentPlayer->is_moving=false;
        display_NBG2((currentPlayer->angle_increment)<<2, -currentPlayer->vertical_angle<<2);
        currentPlayer->angle_increment = currentPlayer->angle_increment/2; //Smoother turn.

        if (currentPlayer->XSP!=0) {currentPlayer->x -= currentPlayer->XSP; currentPlayer->is_moving=true;}  //Bug with integer shift
        //else if (currentPlayer->XSP<0) {currentPlayer->x -= currentPlayer->XSP;currentPlayer->is_moving=true;}

        if (currentPlayer->ZSP!=0){currentPlayer->z -= currentPlayer->ZSP;currentPlayer->is_moving=true;} //Bug with integer shift
        //else if (currentPlayer->ZSP<0){currentPlayer->z -= currentPlayer->ZSP;currentPlayer->is_moving=true;}

        currentPlayer->y += (currentPlayer->YSP>>8);
        currentPlayer->XSP = 0;
        currentPlayer->ZSP = 0;
}

void update_physics(void)
{
    collision_handling(&player);
    update_positions(&player);
}

inline void main_loop(void)
{
    gamepad_control();
    update_physics();
    my_draw();
}



void			jo_main(void)
{
    init_display();
    jo_audio_set_volume(100);
    jo_core_add_callback(main_loop);
	jo_core_run();
}

/*
** END OF FILE
*/
